.intel_syntax noprefix
.altmacro

// For irp the statements are assembled at least once, so
// detecting a no-args condition requires a check for a
// null argument.
//
// The clumsy double-loop arrangement is the only reasonably
// compact method that I could find to push the arguments in
// reverse order.
//
// These macros are coded to minimize the number of loops.
// An .exitm directive in a loop exits the loop only.
//
// These macros work correctly with no optional arguments.
//
// Because the cinvoke macro calls the invoke macro, certain
// types of errors in the cinvoke macro arguments will cause
// errors to be reported for both macros.

.macro invoke, func:req, args:vararg
    n=0
    .irp arg,\args
      .ifnc \arg,
        n=n+1
      .endif
    .endr
    .if n>0
      i=n
      .irp junk,\args
        j=0
        .irp arg,\args
          j=j+1
          .if j==i
            push arg
            .exitm
          .endif
        .endr
        i=i-1
      .endr
    .endif
    call \func
.endm

.macro cinvoke, func:req, args:vararg
    n=0
    .irp arg,\args
      .ifnc \arg,
        n=n+1
      .endif
    .endr
    invoke \func, \args
    .if n>0
      add esp, n*4
    .endif
.endm

